package dk.brics.grammar;

/**
 * Visitor for entities.
 */
public interface EntityVisitor<T> {

	/**
	 * Method to be invoked by {@link Entity#visitBy(EntityVisitor)} on a {@link StringTerminalEntity}.
	 * @param e current entity
	 */
	T visitStringTerminalEntity(StringTerminalEntity e);
	
	/**
	 * Method to be invoked by {@link Entity#visitBy(EntityVisitor)} on a {@link RegexpTerminalEntity}.
	 * @param e current entity
	 */
	T visitRegexpTerminalEntity(RegexpTerminalEntity e);
	
	/**
	 * Method to be invoked by {@link Entity#visitBy(EntityVisitor)} on a {@link EOFTerminalEntity}.
	 * @param e current entity
	 */
	T visitEOFTerminalEntity(EOFTerminalEntity e);
	
	/**
	 * Method to be invoked by {@link Entity#visitBy(EntityVisitor)} on a {@link NonterminalEntity}.
	 * @param e current entity
	 */
	T visitNonterminalEntity(NonterminalEntity e);
}
