package dk.brics.grammar;

/**
 * Production ID object.
 * Used in AST nodes to describe the structure without needing the concrete productions
 * used during parsing (so different grammars can share production IDs).
 */
public class ProductionID {
	
	/** Label, can be null. */
	private String label;
	
	/**
	 * Constructs a new production ID object with null label. 
	 */
	public ProductionID() {}

	/**
	 * Constructs a new production ID object with the given label. 
	 */
	public ProductionID(String label) {
		this.label = label;
	}
	
	/**
	 * Returns the label.
	 * @return label
	 */
	public String getLabel() {
		return label;
	}
	
	/**
	 * Sets the label.
	 * @param label new label
	 */
	public void setLabel(String label) {
		this.label = label;
	}
	
	/**
	 * Checks whether the label is explicit or auto-generated.
	 * @return true if explicit
	 */
	public boolean hasExplicitLabel() {
		return label != null && !label.startsWith("#");
	}
}
