package dk.brics.grammar;

import dk.brics.automaton.Automaton;

/**
 * A regexp terminal.
 */
public class RegexpTerminalEntity extends TerminalEntity {
	
	private Automaton automaton;
	
	/** Maximal match automaton. */
	private boolean max;
	
	/** Title of automaton. */
	private String automaton_name;

	/**
	 * Constructs a new regexp terminal.
	 * @param a automaton representing the regexp
	 * @param max if true, maximal match only
	 * @param aname name of regexp
	 * @param label entity label, null if ignorable
	 * @param example example string, null if absent
	 */
	public RegexpTerminalEntity(Automaton a, boolean max, String aname, String label, String example) {
		super(label, example);
		a.determinize();
		this.automaton = a;
		this.max = max;
		this.automaton_name = aname;
	}
	
	/**
	 * Returns string representation of this entity.
	 */
	@Override
	public String toString() {
		return "<" + automaton_name + ">";
	}

	/**
	 * Returns the automaton of this regexp terminal.
	 * @return the automaton
	 */
	public Automaton getAutomaton() {
		return automaton;
	}

	/**
	 * Returns true if this regexp terminal is maximal match.
	 * @return true if maximal match
	 */
	public boolean isMax() {
		return max;
	}

	/**
	 * Returns the name of this regexp terminal.
	 * @return automaton name
	 */
	public String getAutomatonName() {
		return automaton_name;
	}

	@Override
	public <T> T visitBy(EntityVisitor<T> v) {
		return v.visitRegexpTerminalEntity(this);
	}
}
