package dk.brics.grammar;

/**
 * Basic entity visitor.
 */
public class VoidEntityVisitor implements EntityVisitor<Object> {

	/**
	 * Invokes {@link #visitNonterminal(NonterminalEntity)} and returns null.
	 */
	public final Object visitNonterminalEntity(NonterminalEntity e) {
		visitNonterminal(e);
		return null;
	}

	/**
	 * Invokes {@link #visitTerminal(TerminalEntity)} and 
	 * {@link #visitRegexpTerminal(RegexpTerminalEntity)} and returns null.
	 */
	public final Object visitRegexpTerminalEntity(RegexpTerminalEntity e) {
		visitTerminal(e);
		visitRegexpTerminal(e);
		return null;
	}

	/**
	 * Invokes {@link #visitTerminal(TerminalEntity)} and 
	 * {@link #visitStringTerminal(StringTerminalEntity)} and returns null.
	 */
	public final Object visitStringTerminalEntity(StringTerminalEntity e) {
		visitTerminal(e);
		visitStringTerminal(e);
		return null;
	}

	/**
	 * Invokes {@link #visitTerminal(TerminalEntity)} and 
	 * {@link #visitEOFTerminal(EOFTerminalEntity)} and returns null.
	 */
	public final Object visitEOFTerminalEntity(EOFTerminalEntity e) {
		visitTerminal(e);
		visitEOFTerminal(e);
		return null;
	}

	/**
	 * Empty visitor for {@link NonterminalEntity}.
	 * @param e current entity
	 */
	public void visitNonterminal(NonterminalEntity e) {}

	/**
	 * Empty visitor for {@link TerminalEntity}.
	 * @param e current entity
	 */
	public void visitTerminal(TerminalEntity e) {}

	/**
	 * Empty visitor for {@link RegexpTerminalEntity}.
	 * @param e current entity
	 */
	public void visitRegexpTerminal(RegexpTerminalEntity e) {}

	/**
	 * Empty visitor for {@link StringTerminalEntity}.
	 * @param e current entity
	 */
	public void visitStringTerminal(StringTerminalEntity e) {}

	/**
	 * Empty visitor for {@link EOFTerminalEntity}.
	 * @param e current entity
	 */
	public void visitEOFTerminal(EOFTerminalEntity e) {}
}
