package dk.brics.grammar.ambiguity;

import java.util.ArrayList;
import java.util.List;

import dk.brics.grammar.Entity;
import dk.brics.grammar.Grammar;
import dk.brics.grammar.NonterminalEntity;
import dk.brics.grammar.Production;

public class ExhaustiveSearcher {
	
	private boolean ambiguous;
		
	private Grammar g;
	private ArrayList<ProductionPair<Production, Production>> testedProductions;
	private List <List<Entity>> generatedStrings;
	private List <List<Entity>> lastAddedGeneratedStrings;
	private List <List<Entity>> lastAddedGeneratedStrings2;
	
	/**
	 * create exhaustive searcher
	 * @param g
	 * @param productions
	 */
	public ExhaustiveSearcher(Grammar g, ArrayList<ProductionPair<Production, Production>> productions) {
		this.g = g;
		this.ambiguous = false;
		this.testedProductions = productions;
		this.generatedStrings = new ArrayList<List<Entity>>();
	}
	/**
	 * start exhaustive search on tested productions
	 * @param length
	 */
	public void search(int length) {
		//ked najdem tak pridam do premennej ambiguous true
		Production p1 = testedProductions.get(0).getL();
		Production p2 = testedProductions.get(0).getR();
		if (p1 != null && p2 != null) {
			//TODO I test both rules
		}
		else { //tested only one from horizontal
			//I generate all words for right side
			List<Entity> e = new ArrayList<>();
			e = p1.getEntities();
			generatedStrings.add(e);
			lastAddedGeneratedStrings.add(e);
			while (!lastAddedGeneratedStrings.isEmpty()) {
				for (List<Entity> las : lastAddedGeneratedStrings) {
					for (int i=0; i<las.size(); i++) {
						Entity entity = las.get(i);
						if (entity instanceof NonterminalEntity) {
							for (Production rhs : g.getProductions(((NonterminalEntity) entity).getNonterminal())) {
								List<Entity> newGeneratedString = new ArrayList<>();
								insertToList(las, i, newGeneratedString, rhs.getEntities()); //make new generated string
								if (newGeneratedString.size()<=length) {
										if(lastAddedGeneratedStrings2.contains(newGeneratedString) 
											|| lastAddedGeneratedStrings.contains(newGeneratedString)
											|| generatedStrings.contains(newGeneratedString)) {
										this.ambiguous = true;
									}
									else {
										lastAddedGeneratedStrings2.add(newGeneratedString);
									}
								}
							}
							break; //we rewrite only the most left nonterminal
						}
					}
				}
				//lastADded to generated
				//lastAdded2 to lastAdded
				generatedStrings.addAll(lastAddedGeneratedStrings);
				lastAddedGeneratedStrings.clear();
				lastAddedGeneratedStrings.addAll(lastAddedGeneratedStrings2);
				lastAddedGeneratedStrings2.clear();
			}
		}//end while
	}
	
	private void insertToList(List<Entity> from, int index, List<Entity> where, List<Entity> what) {
		for (int i=0; i<index; i++) {
			where.add(from.get(i));
		}
		for (int i=0; i<what.size(); i++) {
			where.add(what.get(i));
		}
		for (int i=index+1; i<from.size(); i++) {
			where.add(from.get(i));
		}
	}
	
	public boolean isAmbiguous() {
		return ambiguous;
	}
	

}
