package dk.brics.grammar.ambiguity;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;

import com.sun.jndi.cosnaming.CNNameParser;

import dk.brics.automaton.Automaton;
import dk.brics.grammar.EOFTerminalEntity;
import dk.brics.grammar.Entity;
import dk.brics.grammar.EntityVisitor;
import dk.brics.grammar.Grammar;
import dk.brics.grammar.NonterminalEntity;
import dk.brics.grammar.Production;
import dk.brics.grammar.ProductionID;
import dk.brics.grammar.RegexpTerminalEntity;
import dk.brics.grammar.StringTerminalEntity;
import dk.brics.grammar.operations.AutomataOperations;
import dk.brics.grammar.operations.DuplicitProductionRemover;
import dk.brics.grammar.operations.EpsilonProductionRemover;
import dk.brics.grammar.operations.Grammar2JSAGrammar;
import dk.brics.grammar.operations.ProductionLengthNormalizer;
import dk.brics.string.directedgraph.StronglyConnectedComponents;
import dk.brics.string.grammar.AutomatonProduction;
import dk.brics.string.grammar.Nonterminal;
import dk.brics.string.grammar.PairProduction;
//import dk.brics.string.grammar.Production;
import dk.brics.string.grammar.UnitProduction;
import dk.brics.string.grammar.operations.Component;
import dk.brics.string.grammar.operations.Grammar2MLFA;
import dk.brics.string.mlfa.MLFA;
import dk.brics.string.mlfa.operations.MLFA2Automaton;


/**
 * Variation of {@link RegularApproximation} using cycle breaking as is shown in work "Strongly Regular Grammars and Regular Approximation of Context-Free Languages" writed by Omer Egecioglu.
 * @author Jn Ruhalovsk
 */
public class RegularApproximationCycleBreakingDeposit extends RegularApproximation {
	
	private class Pair<L,R> {
	    private L l;
	    private R r;
	    public Pair(L l, R r){
	        this.l = l;
	        this.r = r;
	    }
	    public L getL(){ return l; }
	    public R getR(){ return r; }
	    public void setL(L l){ this.l = l; }
	    public void setR(R r){ this.r = r; }
	}
	
	/**
	 * Cache map from nonterminals to automata.
	 */
	private Map<String,Automaton> nt2a;
	
	/**
	 * Cache map from productions to automata.
	 */
	private Map<Production,Automaton> p2a;
	
	/**
	 * Grammar to MLFA converter.
	 */
	private Grammar2MLFA g2m;
	
	/**
	 * MLFA to automaton converter.
	 */
	private MLFA2Automaton m2a;
	
	/**
	 * Map from nonterminals to JSA grammar nonterminals.
	 */
	private Map<String,dk.brics.string.grammar.Nonterminal> nt_map;
	
	/**
	 * cycles in grammar (nonterminal graph)
	 */
	private StronglyConnectedComponents<Nonterminal, Component> comp;
	
	/**
	 * Constructs a new approximator.
	 */
	public RegularApproximationCycleBreakingDeposit() {}
	
	/**
	 * Returns the name "regularCycleBreaking".
	 */
	@Override
	public String getName() {
		return "regularCycleBreaking";
	}
	
	@Override
	public void init() {
		System.out.println("GRAMMAR: \n" + g.toString());
		
		/*
		 * add new start symbol
		 */
		//g.getNonterminals()
		ArrayList<Entity> ent = new ArrayList<Entity>();
		NonterminalEntity ne = new NonterminalEntity(g.getStart(), null, null);
		ent.add(ne);
		Production newStartProduction = new Production("START", ent, false, new ProductionID(), 0);
		g.addProduction(newStartProduction,true);
		//System.out.println("PRDUCTIONS FOR START " + g.getProductions("START").toString());
		g.setStart(newStartProduction.getNonterminal());
		System.out.println("NOTNERMINALS: " + g.getNonterminals().toString());
		System.out.println("GRAMMAR WITH NEW START SYMBOL: \n" + g.toString());
		
		// normalize grammar
		Grammar normalized = new ProductionLengthNormalizer(out).normalize(g);
		System.out.println("NOTNERMINALS: " + g.getNonterminals().toString());
		System.out.println("NORMALIZED GARMMAR:\n " + normalized.toString() );

		//Epsilon production remove
		EpsilonProductionRemover epr = new EpsilonProductionRemover();
		Grammar removedepsilonGrammar = epr.removeEpsilonProductions(normalized);
		System.out.println("NOTNERMINALS: " + removedepsilonGrammar.getNonterminals().toString() + " SIZE OF PRODUCTIONS: " + removedepsilonGrammar.getProductions().size());
		System.out.println("REMOVED EPSILON GRAMMAR: \n" + removedepsilonGrammar.toString());
		
		//Remove duplicit productions in grammar
		DuplicitProductionRemover dpr = new DuplicitProductionRemover();
		Grammar dprg = dpr.remove(removedepsilonGrammar);
		System.out.println("NOTNERMINALS: " + dprg.getNonterminals().toString() + " SIZE OF PRODUCTIONS: " + dprg.getProductions().size());
		System.out.println("GRAMMAR without duplicit productions: \n" + dprg.toString());
		
		// convert to JSA representation
		Grammar2JSAGrammar conv = new Grammar2JSAGrammar();
		Grammar2JSAGrammar conv2 = new Grammar2JSAGrammar();
		//System.out.println("CONV>\n " + conv.toString() );
		dk.brics.string.grammar.Grammar jsagrammar = conv.convert(dprg);
		nt_map = conv.getNonterminalMap();
		System.out.println("NT_MAP: " + nt_map.values() + "  / KEYS: " + nt_map.keySet());
		dk.brics.string.grammar.Grammar jsagrammarTransformed = conv2.convert(dprg);
		System.out.println("NOTNERMINALS: " + jsagrammar.getNonterminals().toString() + " SIZE OF PRODUCTIONS: " + jsagrammar.getNumberOfProductions());
		System.out.println("JSAG:\n " + jsagrammar.toString() );
		
//		//from JSA to ChomskyNF
//		//TODO GRammar2CNFGrammar class
//		dk.brics.string.grammar.operations.Grammar2CNFGrammar cnfConv = new dk.brics.string.grammar.operations.Grammar2CNFGrammar();
//		//dk.brics.string.grammar.operations.Grammar2CNFGrammar cnfConvForTransform = new dk.brics.string.grammar.operations.Grammar2CNFGrammar();
//		dk.brics.string.grammar.Grammar cnfJsaGrammar = new dk.brics.string.grammar.Grammar();
//		dk.brics.string.grammar.Grammar shiftedCnfGrammar = new dk.brics.string.grammar.Grammar();
//		
//		try {
//			cnfJsaGrammar = cnfConv.convert(jsagrammar);
//			shiftedCnfGrammar = cnfConv.convert(jsagrammarTransformed);
//		} catch (Exception e) {
//			e.printStackTrace();
//		}
		
		dk.brics.string.grammar.Grammar cnfJsaGrammar = jsagrammar;
		dk.brics.string.grammar.Grammar shiftedCnfGrammar = jsagrammarTransformed;
		
		
		//Mohri-Nederhof Transformation
//		System.out.println("NOTNERMINALS: " + cnfJsaGrammar.getNonterminals().toString() + " SIZE OF PRODUCTIONS: " + cnfJsaGrammar.getNumberOfProductions());
//		System.out.println("CNF GRAMMAR BEFORE: " + cnfJsaGrammar.toString());
		//System.out.println("TRANSFORMED GRAMMAR before MN:\n " + transformedCnfJsaGrammar.toString());
		nt_map = conv.getNonterminalMap();
		Map<String,dk.brics.string.grammar.Nonterminal> new_nt_map = conv2.getNonterminalMap();
		System.out.println("NT_MAP: " + nt_map.values() + "  / KEYS: " + nt_map.keySet());
		// run regular approximation and build automaton
		Collection<dk.brics.string.grammar.Nonterminal> hotspots = new ArrayList<dk.brics.string.grammar.Nonterminal>();
		for (String n : g.getNonterminals()) // use all nonterminals from the original grammar as hotspots
			hotspots.add(nt_map.get(n));
		//jsagrammar.approximateNonLinear(hotspots);
		shiftedCnfGrammar.approximateNonLinear(hotspots);
		
		System.out.println("NOTNERMINALS: " + shiftedCnfGrammar.getNonterminals().toString() + " SIZE OF PRODUCTIONS: " + shiftedCnfGrammar.getNumberOfProductions());
		System.out.println("Shifted GRAMMAR after MN:\n " + shiftedCnfGrammar.toString() );
		//System.out.println("CNF GRAMMAR after: " + cnfJsaGrammar.toString());
		
		//System.out.println("pocet neterminalov: " + transformedCnfJsaGrammar.getNonterminals().size());
		shiftedCnfGrammar.shiftKeyValuesForNonterminals(cnfJsaGrammar.getNonterminals().size());//shift keys of noterminals by size of nonterminals from CNFgrammar
		System.out.println("NT_MAP: " + nt_map.values() + "  / KEYS: " + nt_map.keySet());
		System.out.println("NOTNERMINALS: " + shiftedCnfGrammar.getNonterminals().toString() + " SIZE OF PRODUCTIONS: " + shiftedCnfGrammar.getNumberOfProductions());
		System.out.println("Shifted GRAMMAR after SHIFTING:\n " + shiftedCnfGrammar.toString() );
		//System.out.println("PRVY: " + transformedCnfJsaGrammar.getNonterminals().get(0).getKey());
		//System.out.println("POSEDNY: " + transformedCnfJsaGrammar.getNonterminals().get(transformedCnfJsaGrammar.getNonterminals().size()-1).getKey());
		
		cnfJsaGrammar.addProductionsFromGrammar(shiftedCnfGrammar);
		
		System.out.println("NOTNERMINALS: " + cnfJsaGrammar.getNonterminals().toString() + " SIZE OF PRODUCTIONS: " + cnfJsaGrammar.getNumberOfProductions());
		System.out.println("Added Shifted rules to cnfGRam: \n" + cnfJsaGrammar.toString());
		
		
		System.out.println("NT_MAP: " + nt_map.values() + "  / KEYS: " + nt_map.keySet());
		
		//Find Cycles on CNF grammar
		//ArrayList<Pair<Nonterminal, dk.brics.string.grammar.Production>> breaketedProductionsLeft = new ArrayList<Pair<Nonterminal, dk.brics.string.grammar.Production>>(); 
		//ArrayList<Pair<Nonterminal, dk.brics.string.grammar.Production>> breaketedProductionsRight = new ArrayList<Pair<Nonterminal, dk.brics.string.grammar.Production>>(); 
		boolean allMonochromatic = false;
		while (!allMonochromatic) {
			allMonochromatic = true;
			comp = cnfJsaGrammar.getComponents(true);
			System.out.println("==========================ITERATION========================================");
			for (Component c : comp.getComponents()) {
				//c.findRecursion();
				System.out.println("COMPONENTS: " + c.toString() + " SIZE: " + c.getNodes().size() + " RECURSION: " + c.getRecursion().name() + "Size of Productions for 1. element: " + c.getNode(0).getProductions().size());
				boolean left = false;
				int leftCount = 0;
				boolean right = false;
				int rightCount = 0;
				System.out.println("RECURSION: " + c.getRecursion().name());
				if (c.getRecursion().name().equals("BOTH")) {
					int oldKey = -1;
					int shiftNum = -1;
					allMonochromatic = false;
					System.out.println("BOTH RECURSION: " + c.getNodes());
					for ( dk.brics.string.grammar.Production p : cnfJsaGrammar.getNonterminal(c.getNode(0).getKey()).getProductions() ) {
						if (p instanceof PairProduction) {
							if (c.getNodes().contains(((PairProduction) p).getNonterminal1())) {
								oldKey = ((PairProduction) p).getNonterminal1().getKey();
								shiftNum = oldKey + shiftedCnfGrammar.getShiftKeyValue();
								System.out.println("SHIFTED NUBER: " + shiftNum);
								((PairProduction) p).setNonterminal1(shiftedCnfGrammar.getNonterminal(shiftNum));
								break;
							}
						}
						if (p instanceof PairProduction) {
							if (c.getNodes().contains(((PairProduction) p).getNonterminal2())) {
								oldKey = ((PairProduction) p).getNonterminal2().getKey();
								shiftNum = oldKey + shiftedCnfGrammar.getShiftKeyValue();
								System.out.println("SHIFTED NUBER: " + shiftNum);
								((PairProduction) p).setNonterminal2(shiftedCnfGrammar.getNonterminal(shiftNum));
								break;
							}
						}
					}
				}
//				if (c.getNodes().size()>1) { //here is cycle, but i dont know wheter monochromatic or NONmonochromatic
//					for (Nonterminal n : c.getNodes()) {
//						for (Production p : n.getProductions()) {
//							if (p instanceof PairProduction) {
//								// if left nonterminal make cycle, then is left cycle
//								if ( c.getNodes().contains(((PairProduction) p).getNonterminal1()) ) {
//									left = true;
//									leftCount++;
//								}
//								//if right nonterminal make cycle, then is right cycle
//								if ( c.getNodes().contains(((PairProduction) p).getNonterminal2()) ) {
//									right = true;
//									rightCount++;
//								}
//							}
//							else if (p instanceof UnitProduction) {
//								//ignore
//							}
//							else if (p instanceof AutomatonProduction) {
//								
//							}
//							else {
//								try {
//									throw new Exception("grammar is not in Chomsky Normal Form!");
//								} catch (Exception e) {
//									e.printStackTrace();
//								}
//							}
//							cnfJsaGrammar.getNonterminal(c.getNodes().toString());
//						}
//					}
//					if ((left && !right) || (!left && right)) {
//						//System.out.println("CYCLE/COMPONENT " + c.toString() + " is monochromatic");
//					}
//					else {
//						//System.out.println("CYCLE/COMPONENT " + c.toString() + " is NONmonochromatic");
//						allMonochromatic = false;
//						int index = -1;
//						boolean haveLeftRec = false;
//						boolean haveRightRec = false;
//						for (Production p : cnfJsaGrammar.getNonterminal(c.getNode(0)).getProductions()) {
//							index++;
//							if (p instanceof PairProduction) {
//								if ( c.getNodes().contains(((PairProduction) p).getNonterminal1()) ) {
//									//cnfJsaGrammar.getNonterminal(c.getNode(0)).getProductions().clear();
//									haveLeftRec = true;
//									break;
//								}
//								else if ( c.getNodes().contains(((PairProduction) p).getNonterminal2()) ) {
//									haveRightRec = true;
//									break;
//								}
//							}
//						}
//						if (haveLeftRec) {
//							//remove production with recursion
//							Nonterminal rn;
//							rn = cnfJsaGrammar.getNonterminal(c.getNode(0));
//							Production rp;
//							rp = rn.getProductions().remove(index);
//							System.out.println("REMOVED: " +rn.toString() + "->" + rp.toString());
//							breaketedProductionsLeft.add(new Pair<Nonterminal, Production>(rn, rp));
//							//add reference to terminal in other/transformed grammar
//							if (rp instanceof PairProduction) {
//								//((PairProduction) rp).getNonterminal1().se
//								//System.out.println("NON" + transformedCnfJsaGrammar.getNonterminal(rn.getKey()+transformedCnfJsaGrammar.getShiftKeyValue()).toString());
//								//cnfJsaGrammar.addPairProduction(rn, transformedCnfJsaGrammar.getNonterminal(((PairProduction) rp).getNonterminal1().getKey()+transformedCnfJsaGrammar.getShiftKeyValue()), ((PairProduction) rp).getNonterminal2());
//							}
//						}
//						if (haveRightRec) {
//							//remove production with recursion
//							Nonterminal rn;
//							rn = cnfJsaGrammar.getNonterminal(c.getNode(0));
//							Production rp;
//							rp = rn.getProductions().remove(index);
//							System.out.println("REMOVED: " +rn.toString() + "->" + rp.toString());
//							breaketedProductionsRight.add(new Pair<Nonterminal, Production>(rn, rp));
//							//add reference to terminal in other/transformed grammar
//							if (rp instanceof PairProduction) {
//								//((PairProduction) rp).getNonterminal1().se
//								//cnfJsaGrammar.addPairProduction(rn, ((PairProduction) rp).getNonterminal1(), transformedCnfJsaGrammar.getNonterminal(rn.getKey()+transformedCnfJsaGrammar.getShiftKeyValue()));
//							}
//						}
//					}
//				}
			}
			//System.out.println("BREAKING: \n" + cnfJsaGrammar.toString());
		}
//		for (int i=0; i<breaketedProductions.size(); i++) {
//			System.out.println(breaketedProductions.get(i).getL().toString() + " STRANA: " + breaketedProductions.get(i).getR());
//		}
		
		/*
		for (int i=0; i<breaketedProductionsLeft.size(); i++) {
			Nonterminal n = breaketedProductionsLeft.get(i).getL();
			dk.brics.string.grammar.Production p = breaketedProductionsLeft.get(i).getR();
			if (p instanceof PairProduction) {
				cnfJsaGrammar.addPairProduction(n, shiftedCnfGrammar.getNonterminal(((PairProduction) p).getNonterminal1().getKey()+shiftedCnfGrammar.getShiftKeyValue()), ((PairProduction) p).getNonterminal2());
			}
		}
		for (int i=0; i<breaketedProductionsRight.size(); i++) {
			Nonterminal n = breaketedProductionsRight.get(i).getL();
			dk.brics.string.grammar.Production p = breaketedProductionsRight.get(i).getR();
			if (p instanceof PairProduction) {
				cnfJsaGrammar.addPairProduction(n, ((PairProduction) p).getNonterminal1(), shiftedCnfGrammar.getNonterminal(((PairProduction) p).getNonterminal2().getKey()+shiftedCnfGrammar.getShiftKeyValue()));
			}
		}
		*/
		System.out.println("MONOCHROMATIC: \n" + cnfJsaGrammar.toString());
		//comp = cnfJsaGrammar.getComponents();
		//System.out.println("TRANSFORMED: \n" + transformedCnfJsaGrammar.toString());
		
		//dk.brics.string.grammar.Grammar resultGram = new dk.brics.string.grammar.Grammar();
		
		
		
		//cnfJsaGrammar.addProductionsFromGrammar(transformedCnfJsaGrammar);
		//System.out.println("RESULT GRAMMAR: \n" + resultGram.toString());
		System.out.println("RESULT GRAMMAR: \n" + cnfJsaGrammar.toString());
		
		
		g2m = new Grammar2MLFA(cnfJsaGrammar);
		MLFA mlfa = g2m.convert();
		System.out.println(mlfa.toString());
		System.out.println("MLFA: " + mlfa.toString());
		/*
		if (debug)
			out.print("mlfa:\n" + mlfa.toString());
		*/
		System.out.println("NT_MAP: " + nt_map.values());
		m2a = new MLFA2Automaton(mlfa);
		System.out.println("TU SOM");
		nt2a = new HashMap<String,Automaton>();
		clearProductionCache();
		System.out.println("JOJO SIZE: " + nt_map.size() + "DALI SIZE: " + new_nt_map.size());
//		System.out.println("NEW_NT_MAP: " + new_nt_map.values());
////		for (int i=0; i<new_nt_map.size(); i++) {
////			nt_map.put(new_nt_map.get(i).toString(), conv2.getNonterminalMap().get(i));
////			//nt_map.putAll(conv2.getNonterminalMap());
////		}
//		nt_map.putAll(new_nt_map);
////		Iterator<Entry<String, dk.brics.string.grammar.Nonterminal>> it = new_nt_map.entrySet().iterator();
////		while(it.hasNext()) {
////			Map.Entry pairs = (Map.Entry) it.next();
////			nt_map.put(pairs.getKey(), pairs.getValue());
////		}
//		System.out.println("NT_MAP: " + nt_map.values());
//		System.out.println("NEW_NT_MAP: " + new_nt_map.values());
//		
//		
////		Map<String,dk.brics.string.grammar.Nonterminal> new_nt_map = new TreeMap<String, Nonterminal>();
////		new_nt_map.putAll(nt_map);
////		for (int i=0; i<nt_map.size(); i++) {
////			Nonterminal n = nt_map.get(i); 
////			new_nt_map.put(n.toString(), n);
////		}
//		
////		for (Nonterminal n : cnfJsaGrammar.getNonterminals()) {
////			if ( !nt_map.containsValue(n) ) {
////				nt_map.put(n.toString(), n);
////			}
////		}
	}
	
	private void clearProductionCache() {
		p2a = new HashMap<Production,Automaton>();
	}
	
	private Automaton getAutomaton(String n) {
		System.out.println("getAutomaton for String : " + n);
		Automaton a = nt2a.get(n);
		if (a == null) {
			System.out.println("NT_MAP.get(n) : " + nt_map.get(n).toString() + "  /  MLFAstatePAIR: " + g2m.getMLFAStatePair(nt_map.get(n)).toString());
			a = m2a.extract(g2m.getMLFAStatePair(nt_map.get(n)));
			nt2a.put(n, a);
		}
		return a;
	}
	
	private Automaton getAutomaton(dk.brics.grammar.Production p) {
		System.out.println("getAutomaton for Production : " + p.toString());
		Automaton a = p2a.get(p);
		if (a == null) {
			a = getAutomaton(p.getEntities(), 0, p.getEntities().size());
			p2a.put(p, a);
			p2a.put(p, a);
		}
		return a;
	}
	
	private Automaton getAutomaton(List<Entity> entities, int from, int to) { // FIXME: handle unordered productions
		List<Automaton> as = new ArrayList<Automaton>();
		for (int i = from; i < to; i++) {
			Entity e = entities.get(i);
			Automaton a = e.visitBy(new EntityVisitor<Automaton>() {
				public Automaton visitNonterminalEntity(NonterminalEntity e) {
					return getAutomaton(e.getNonterminal());
				}
				public Automaton visitRegexpTerminalEntity(RegexpTerminalEntity e) { // FIXME: handle MAX regexps
					return e.getAutomaton();
				}
				public Automaton visitStringTerminalEntity(StringTerminalEntity e) {
					return Automaton.makeString(e.getString());
				}
				public Automaton visitEOFTerminalEntity(EOFTerminalEntity e) {
					return Automaton.makeEmptyString(); // FIXME: sound to treat EOF as "" - but not ideal!
				}
			});
			as.add(a);
		}		
		Automaton aut = Automaton.concatenate(as);
		//aut.minimize(); // TODO: need minimization here?
		return aut;
	}
	
	@Override
	public HorizontalOverlapString checkHorizontalOverlap(dk.brics.grammar.Production p, int index) { // FIXME: handle the case where exactly one of the two surrounding entities is ignorable
		System.out.println("check HORIYONTAL: ");
		//return super.checkHorizontalOverlap(p, index);
		Automaton a1 = getAutomaton(p.getEntities(), 0, index);
		Automaton a2 = getAutomaton(p.getEntities(), index, p.getEntities().size());
		Automaton overlap = AutomataOperations.getOverlap(a1, a2);
		// TODO: would it help to intersect overlap with p? (dangling-else...)
		return AutomataOperations.getOverlapString(overlap);
	}

	@Override
	public VerticalOverlapString checkVerticalOverlap(dk.brics.grammar.Production p1, dk.brics.grammar.Production p2) {
		//return super.checkVerticalOverlap(p1, p2);
		System.out.println("-------------check VERTICALL------------------------ ");
		System.out.println("check VERTICALL: ");
		
		System.out.println("AUTOMATON 1:");
		Automaton a1 = getAutomaton(p1);
		System.out.println("A1: " + a1.toString());
		
		System.out.println("AUTOMATON 2:");
		Automaton a2 = getAutomaton(p2);
		System.out.println("A2: " + a2.toString());
		
		Automaton overlap = a1.intersection(a2); // TODO: make combined operation for intersection+getShortestExample
		System.out.println("overlap");
		// TODO: would it help to intersect overlap with p1/p2?
		String s = overlap.getShortestExample(true);
		System.out.println("String:" + s);
		return (s != null) ? new VerticalOverlapString(s) : null;
	}

}
