package dk.brics.grammar.ast;

/**
 * Abstract syntax tree (AST).
 */
public class AST {
	
	/** Root node. */
	private BranchNode root;

	/** Contents string. (Substrings are relative to this string.) */
	private String x;

	/**
	 * Constructs a new AST.
	 * @param root root node
	 * @param x original string
	 */
	public AST(BranchNode root, String x) {
		this.root = root;
		this.x = x;
	}
	
	/**
	 * Returns the root node.
	 * @return root node
	 */
	public BranchNode getRoot() {
		return root;
	}

	/**
	 * Returns the original string.
	 * @return original string
	 */
	public String getOriginalString() {
		return x;
	}

	/**
	 * Returns string representation of this AST.
	 */
	@Override
	public String toString() {
		StringBuilder b = new StringBuilder();
		root.print(x, b);
		return b.toString();
	}
	
	/**
	 * Traverses this AST bottom-up and applies the visitor to all nodes.
	 * @param v AST visitor
	 */
	public void traverse(NodeVisitor v) {
		root.traverse(v);
	}
}
