package dk.brics.grammar.ast;

import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeSet;

import static dk.brics.misc.Chars.escape;

// TODO: parse tree diff (colored)

/**
 * Printer for <a href="http://www.research.att.com/sw/tools/graphviz/" 
 * target="_top">Graphviz Dot</a> format.
 */
public class DotPrinter {

	private PrintWriter p;
	
	/**
	 * Constructs a new dot printer.
	 * @param p print writer for output
	 */
	public DotPrinter(PrintWriter p) {
		this.p = p;
	}
	
	/**
	 * Prints the given AST.
	 * @param t AST
	 */
	public void print(final AST t) {
		p.println("digraph AST {");
		t.traverse(new NodeVisitor() {

			private Map<Node,Integer> nodekey = new HashMap<Node,Integer>();
			private int next_key = 0;

			public void visitLeafNode(LeafNode s) {
				p.println("  " + id(s) + " [shape=plaintext,label=\"" + escape(s.getString(t.getOriginalString()), true) + "\"];");
			}

			public void visitBranchNode(BranchNode n) {
				String l = escape(n.getName(), true);
				String label = n.getProductionID().getLabel();
				if (label != null)
					l += "\\[" + label + "\\]";
				p.println("  " + id(n) + " [shape=box,label=\"" + l + "\"];");
				for (String s : new TreeSet<String>(n.getChildNames())) {
					Node q = n.getChild(s);
					p.println("  " + id(n) + " -> " + id(q) + " [label=\"" + escape(s, true) + "\"];");
				}
			}
			
			private String id(Node t) {
				Integer key = nodekey.get(t);
				if (key == null) {
					key = next_key++;
					nodekey.put(t, key);
				}
				return "n" + key;
			}
		});
		p.println("}");
	}
}
