package dk.brics.grammar.ast;

/**
 * Leaf node, representing a substring of the original string.
 */
public class LeafNode extends Node {
	
	private String override;
	
	/**
	 * Constructs a new leaf node.
	 * @param i from index
	 * @param j to index (not included)
	 */
	public LeafNode(int i, int j) {
		super(i, j);
	}

	@Override
	public void print(String x, StringBuilder b) {
		b.append(getString(x));
	}

	@Override 
	void traverse(NodeVisitor v) {
		v.visitLeafNode(this);
	}

	@Override 
	public void visitBy(NodeVisitor v) {
		v.visitLeafNode(this);
	}
	
	/**
	 * Sets the string. (Overrides the original substring.)
	 * @param s new string
	 */
	public void setString(String s) {
		override = s;
	}

	/**
	 * Returns the substring represented by this node.
	 * @param x original string (irrelevant if substring has been set by {@link #setString(String)}})
	 * @return substring
	 */
	@Override
	public String getString(String x) {
		if (override != null)
			return override;
		return super.getString(x);
	}
}
