package dk.brics.grammar.ast;

/**
 * Superclass of AST nodes.
 */
public abstract class Node {
	
	/** Start index in contents string. */
	private int i;
	
	/** End index (exclusive) in contents string. */
	private int j;
	
	/**
	 * Constructs a new node.
	 * @param i from index
	 * @param j to index
	 */
	protected Node(int i, int j) {
		this.i = i;
		this.j = j;
	}
	
	/**
	 * Prints this node.
	 * @param x original string
	 * @param b string buffer for output
	 */
	abstract public void print(String x, StringBuilder b);
	
	/**
	 * Returns index in original string corresponding to the start of this node.
	 * @return index
	 */
	public int getFromIndex() {
		return i;
	}
	
	/**
	 * Returns index in original string corresponding to the end (exclusive) of this node.
	 * @return index
	 */
	public int getToIndex() {
		return j;
	}
	
	/**
	 * Returns the substring represented by this node.
	 * @param x original string
	 * @return substring
	 */
	public String getString(String x) {
		return x.substring(i, j);
	}
	
	abstract void traverse(NodeVisitor v);
	
	/**
	 * Visits this node.
	 * @param v visitor
	 */
	public abstract void visitBy(NodeVisitor v);
}
