package dk.brics.grammar.main;

import java.awt.*;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.*;

/**
 * Image viewer.
 */
public class ImageViewer {
	
	private ImageViewer() {}
	
	/**
	 * Opens an image viewer window.
	 * @param title window title
	 * @param path path to image file
	 * @return true if successful, false if IO error occurred
	 */
	public static boolean open(String title, String path) {
		try {
			new ImageFrame(title, ImageIO.read(new File(path)), path).setVisible(true);
		} catch (IOException e) {
			return false;
		}
		return true;
	}
	
	static class ImageFrame extends JFrame { // TODO: zoom, save, ...
		
		Image img;
		String path;
		int width;
		int height;
		
		ImageFrame(String title, Image img, String path) {
			super(title);
			this.img = img;
			this.path = path;
			getContentPane().setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));
			ImagePanel imagepanel = new ImagePanel();
			width = img.getWidth(null);
			height = img.getHeight(null);
			imagepanel.setPreferredSize(new Dimension(width, height));
			JScrollPane scrollpane = new JScrollPane(imagepanel);
			scrollpane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
			scrollpane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
			scrollpane.setPreferredSize(new Dimension(400, 400));
			add(scrollpane);
			pack();
		}
		
		@Override
		public void setVisible(boolean b) {
		  if (b) {
			  Dimension screensize = Toolkit.getDefaultToolkit().getScreenSize();
			  setLocation((screensize.width-getWidth())/2, (screensize.height-getHeight())/2);
		  }
		  super.setVisible(b);
		}

		class ImagePanel extends JPanel {
			@Override
			public void paint(Graphics g) {
				super.paint(g);
				g.drawImage(img, 0, 0, width, height, this);
			}
		}
	}
	
	/*
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			open("/users/amoeller/test.png");
		} catch (Exception e) {
			System.err.println(e.getMessage());
		}
	}
	*/
}
