package dk.brics.grammar.main;

import java.io.IOException;
import java.io.PrintWriter;
import java.nio.charset.Charset;
import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;

import dk.brics.misc.Loader;

/**
 * Servlet for parsing and grammar ambiguity checking.
 * @see Main
 */
public class MainServlet extends HttpServlet {
	
	/**
	 * Constructs a new servlet.
	 */
	public MainServlet() {
		super();
	}

	@SuppressWarnings("unchecked")
	@Override
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException {
		if (!ServletFileUpload.isMultipartContent(request)) {
			response.sendError(HttpServletResponse.SC_BAD_REQUEST);
			return;
		}
		String mode =  "check";
		String grammar = "";
		String grammarurl = "";
		String text = "";
		String texturl = "";
		boolean verbose = false;
		boolean dumpast = false;
		boolean tokenize = false;
		String grammar_location = null;
		String text_location = null;
		String grammar_encoding, text_encoding;
		String unfold_left, unfold_right;
		int unfold_level = 0;
		unfold_left = unfold_right = null;
		grammar_encoding = text_encoding = Charset.defaultCharset().name();

		DiskFileItemFactory factory = new DiskFileItemFactory();
		factory.setSizeThreshold(100000);
		ServletFileUpload upload = new ServletFileUpload(factory);
		upload.setSizeMax(10000);
		List items;
		try {
			items = upload.parseRequest(request);
		} catch (FileUploadException e) {
			response.sendError(HttpServletResponse.SC_BAD_REQUEST, e.getMessage());
			return;
		}
		Iterator i = items.iterator();
		while (i.hasNext()) {
		    FileItem item = (FileItem)i.next();
		    if (item.isFormField()) {
		    	String name = item.getFieldName();
		        String value = item.getString();
		        if (name.equals("mode")) 
		        	mode = value;
		        else if (name.equals("grammar"))
		        	grammar = value;
		        else if (name.equals("grammarurl"))
		        	grammarurl = value.trim();
		        else if (name.equals("text"))
		        	text = value;
		        else if (name.equals("texturl"))
		        	texturl = value.trim();
		        else if (name.equals("unfold_level"))
		        	try {
		        		unfold_level = Integer.parseInt(value.trim());
		        	} catch (NumberFormatException e) {
		        		unfold_level = 0;
		        	}
		        else if (name.equals("unfold_left"))
		        	unfold_left = value;
		        else if (name.equals("unfold_right"))
		        	unfold_right = value;
		        else if (name.equals("grammarencoding") && !value.equals("default"))
		        	grammar_encoding = value;
		        else if (name.equals("textencoding") && !value.equals("default"))
		        	text_encoding = value;
		        else if (name.equals("verbose"))
		        	verbose = true;
		        else if (name.equals("dumpast"))
		        	dumpast = true;
		        else if (name.equals("tokenize"))
		        	tokenize = true;
		    }
		}
		if (unfold_level < 0)
			unfold_level = 0;
		i = items.iterator();
		while (i.hasNext()) {
		    FileItem item = (FileItem)i.next();
		    if (!item.isFormField() && item.getName().length() > 0) {
		    	String name = item.getFieldName();
		        if (name.equals("grammarupload")) {
		        	grammar = item.getString(grammar_encoding);
		        	grammar_location = item.getName();
		        } else if (name.equals("textupload")) {
		        	text = item.getString(text_encoding); 
		        	text_location = item.getName();
		        } 
		    }
		}
		if (grammarurl.length() > 0) {
			if (!grammarurl.startsWith("http://")) {
				response.sendError(HttpServletResponse.SC_BAD_REQUEST, "illegal URL");
				return;
			}
			grammar = Loader.getString(grammarurl, grammar_encoding);
			grammar_location = grammarurl;
		} else if (grammar_location == null) 
			grammar_location = "[input grammar]";
		if (mode.equals("parse")) {
			if (texturl.length() > 0) {
				if (!texturl.startsWith("http://")) {
					response.sendError(HttpServletResponse.SC_BAD_REQUEST, "illegal URL");
					return;
				}
				text = Loader.getString(texturl, text_encoding);
				text_location = texturl;
			} else if (text_location == null)
				text_location = "[input text]";
		}
		
		String client = request.getHeader("x-forwarded-for");
		if (client == null)
			client = request.getRemoteHost();
		log("request from " + client + ": " + mode + "," + grammar_location + "," + grammar.length() + "," + 
				text_location + "," + text.length() + "," + "," + verbose + "," + dumpast + "," + tokenize);
		
		response.setDateHeader("Expires", 0);
	    response.setHeader("Cache-Control", "no-store, no-cache, must-revalidate");
	    response.setHeader("Pragma", "no-cache");
	    
	    // TODO: run graphviz dot if dumpast
	    
	    response.setContentType("text/html");
	    response.setCharacterEncoding("ISO-8859-1");
	    HtmlEscapingPrintWriter out = new HtmlEscapingPrintWriter(response.getWriter());
	    out.uprintln("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">" +
	    		"<html><head><meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\">" +
	    		"<title>dk.brics.grammar</title></head></body>");	    
	    /*
	    out.uprint("<h1>input grammar:</h1><pre>");
	    out.uprintln(htmlEscape(grammar));
	    if (text_location != null) {
		   	out.uprint("</pre><h1>input text:</h1><pre>");
		   	out.uprintln(htmlEscape(text));
	    }
	    out.uprintln("</pre>");
	    */

		out.uprint("<h1>dk.brics.grammar reply:</h1><pre>");
	    try {
	    	TimeoutThread timer = new TimeoutThread(10);
	    	try {
	    		Main.run(grammar, grammar_location, text, text_location, mode.equals("analyze"), 
	    				unfold_level, unfold_left, unfold_right, verbose, dumpast, tokenize, false, out); // TODO: memory limit?
	    	} finally {
	    		timer.cancel();					
	    	}
	    } catch (IllegalArgumentException e) {
	    	out.println("input error: " + e.getMessage());
	    	Thread.interrupted();
	    } catch (Throwable t) {
	    	log(t.getMessage());
	    	out.println("timeout, execution aborted");
	    	Thread.interrupted();
	    }
	    
	    out.uprintln("</pre><form name=\"back\" action=\"\"><input type=button value=\"Back to the demo\" onClick=\"history.go(-1)\">" +
	    		"</form></body></html>");
	    out.close();
	}

	private static String htmlEscape(String s) {
		StringBuffer b = new StringBuffer();
		for (int i = 0; i<s.length(); i++) {
			char c = s.charAt(i);
			switch (c) {
			case '<': b.append("&lt;"); break;
			case '>': b.append("&gt;"); break;
			case '"': b.append("&quot;"); break;
			case '&': b.append("&amp;"); break;
			default: b.append(c);
			}
		}
		return b.toString();
	}
	
	static class HtmlEscapingPrintWriter extends PrintWriter {

		public HtmlEscapingPrintWriter(PrintWriter out) {
			super(out, true);
		}

		@Override
		public void print(String s) {
			super.print(htmlEscape(s));
		}
		
		public void uprint(String s) {
			super.print(s);
		}

		public void uprintln(String s) {
			super.print(s);
			super.println();
		}
	}
}
