package dk.brics.grammar.operations;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import dk.brics.automaton.Automaton;
import dk.brics.automaton.State;
import dk.brics.automaton.Transition;
import dk.brics.grammar.Entity;
import dk.brics.grammar.Grammar;
import dk.brics.grammar.NonterminalEntity;
import dk.brics.grammar.Production;
import dk.brics.grammar.ProductionID;
import dk.brics.grammar.RegexpTerminalEntity;
import dk.brics.string.Misc;

/**
 * Converter from automata to regular grammars.
 * The resulting grammars contain only one-transition-interval terminal automata.
 */
public class Automaton2Grammar {
	
	private int next_key = 1;
	
	/**
	 * Constructs a new converter.
	 */
	public Automaton2Grammar() {}
	
	/**
	 * Converts the given automaton to a new grammar.
	 * @param a automaton
	 * @return grammar
	 * @see #extend(Grammar, Automaton)
	 */
	public Grammar convert(Automaton a) {
		Grammar g = new Grammar(null, new ArrayList<Production>());
		g.setStart(extend(g, a));
		return g;
	}
	
	/**
	 * Extends an existing grammar with a right linear sub-grammar corresponding to a given automaton.
	 * If invoking this on multiple automata, extending the same grammar, then
	 * the same <code>Automaton2Grammar</code> converter should be used.
	 * All new productions and entities are non-labeled.
	 * @param g existing grammar
	 * @param a automaton to convert
	 * @return start nonterminal of the new sub-grammar 
	 */
	public String extend(Grammar g, Automaton a) {
		Map<State,String> state2nt = new HashMap<State,String>();
		Set<State> states = a.getStates();
		System.out.println(a);
		for (State s : states) {
			String nt = "@" + next_key++;
			state2nt.put(s, nt);
		}
		Collection<Production> newprods = new ArrayList<Production>();
		for (State s : states) {
			String nt = state2nt.get(s);
			if (s.isAccept())
				newprods.add(new Production(nt, new ArrayList<Entity>(), false, new ProductionID(), 0));
			for (Transition t : s.getSortedTransitions(false)) {
				List<Entity> entities = new ArrayList<Entity>();
				String aname = Misc.escape(Character.toString(t.getMin()));;
				if (t.getMin() < t.getMax())
					aname = aname + "-" + Misc.escape(Character.toString(t.getMax()));
				entities.add(new RegexpTerminalEntity(Automaton.makeCharRange(t.getMin(), t.getMax()), false, aname, null, null));
				entities.add(new NonterminalEntity(state2nt.get(t.getDest()), null, null));
				newprods.add(new Production(nt, entities, false, new ProductionID(), 0));
			}
		}
		g.addProductions(newprods);
		return state2nt.get(a.getInitialState());
	}
}
