package dk.brics.grammar.operations;

import java.util.ArrayList;
import java.util.LinkedList;

import dk.brics.grammar.Entity;
import dk.brics.grammar.Grammar;
import dk.brics.grammar.Production;
import dk.brics.grammar.ProductionID;

/**
 * 
 * @author Jn Ruhalovsk
 *
 */
public class EpsilonProductionRemover {
		
	/** Epsilon Productions. */
	private LinkedList<Production> epsilonProductions;
	
	/** 
	 * Constructs a new epsilon produtions remover (Remover removes all epsilon productions from grmmar).
	 * @param out print writer for messages (if a grammar contains unordered productions)
	 */
	public EpsilonProductionRemover() {
		this.epsilonProductions = new LinkedList<Production>();
	}
	
	/**
	 * Modify the given normalized grammar to form without epsilon productions.
	 * The input grammar is not modified, but the output grammar reuses the <code>Entity</code> objects from the input grammar.
	 * @param ng input grammar
	 * @return output grammar
	 */
	public Grammar removeEpsilonProductions(Grammar ng) {
		LinkedList<Production> productions = new LinkedList<Production>();
		LinkedList<Production> usedEpsilonProduction = new LinkedList<Production>();
		
		/*
		 * FIND epsilon productions 
		 */
		for (Production p : ng.getProductions()) {			
			//if p is epsilon production AND left hand side is not start nonterminal
			if (p.getEntities().size()==0 && !ng.getStart().equals(p.getNonterminal())) {
				//System.out.println("EMPTY");
				epsilonProductions.add(p);
			}
			else {
				productions.add(p);
			}
			//System.out.println("EPSILON PRODUCTIONS: " + epsilonProductions.toString());
			//System.out.println("PRODUCTIONS: " + productions.toString());
		}
		
		//VYPIS
//		System.out.println("PRODUCTIONS:");
//		for (Production p : productions) {
//			System.out.println("P: " + p.print(p.getEntities()));
//		}
//		System.out.println("EpsilonProductions");
//		for (Production p : epsilonProductions) {
//			System.out.println("EPSILON PROD: " + p.print(p.getEntities()));
//		}
		
		//new added productions to productions set
		LinkedList<Production> addedProductions = new LinkedList<Production>();
		//HashSet<Production> addedProductions = new HashSet<Production>();
		
		/*
		 * remove epsilon porduction from grammar
		 */
		while (!epsilonProductions.isEmpty()) {
			//System.out.println("EPS PROD SIZE: " + epsilonProductions.size());
			//System.out.println("PROD:" + epsilonProductions.get(0).print(epsilonProductions.get(0).getEntities()));
			Production ep = epsilonProductions.peekFirst();
			//notEpsP = not epsilon production
			
			//for (int i=0; i<productions.size(); i++) {
			for (Production notEpsP : productions) {
				int priority = notEpsP.getPriority();
				if (notEpsP.getEntities().size() == 0 && notEpsP.getNonterminal().equals(ng.getStart())) {
					//is start epsilon production
				}
				else if (notEpsP.getEntities().size() == 1) {
					//System.out.println("EP: " + ep.getNonterminal());
					//System.out.println("NOTEP> " + notEpsP.getEntities().get(0).toString());
					if (notEpsP.getEntities().get(0).toString().equals(ep.getNonterminal())) {
						//make new epsilon production
						Production newP = new Production(notEpsP.getNonterminal(), new ArrayList<Entity>(), false, new ProductionID()/*new ProductionID(label)*/, priority);
						boolean contains = false;
						for (Production tmpEp: epsilonProductions) {
							if (tmpEp.equalsTo(newP)) {
								contains = true;
								break;
							}
						}
						if (!contains && !usedEpsilonProduction.contains(newP)/*&& !newP.getNonterminal().equals(ng.getStart())*/) {
							//System.out.println("NEWP.GETNONTERMINAL: " + newP.getNonterminal());
							//System.out.println("NG.GETSTART: " + ng.getStart());
							epsilonProductions.add(newP);
							System.out.println("ADDED EMPTY: " + newP.print(newP.getEntities()));
						}
						//notEpsP.setEntities(null);
						//removedProd.add(notEpsP);
						System.out.println("VELKOST 1 " + notEpsP.print(notEpsP.getEntities()));
					}
					else {
						//on the right side is not epsiloned nonterminal
					}
				}
				else if (notEpsP.getEntities().size() == 2) {
					//if both Nonterminal on the right side are equal (A -> B B)
					if (notEpsP.getEntities().get(0).toString().equals(ep.getNonterminal())
							&& notEpsP.getEntities().get(1).toString().equals(ep.getNonterminal())) {
						//make new epsilon production
						Production newP = new Production(notEpsP.getNonterminal(), new ArrayList<Entity>(), false, new ProductionID()/*new ProductionID(label)*/, priority);
						//System.out.println("NEW P: " + newP.print(newP.getEntities()));
						boolean contains = false;
						for (Production tmpEp: epsilonProductions) {
							if (tmpEp.equalsTo(newP)) {
								contains = true;
								break;
							}
						}
						//create new epsilon porduction
						if (!contains && !usedEpsilonProduction.contains(newP) /*&& !newP.getNonterminal().equals(ng.getStart())*/) {
							epsilonProductions.add(newP);
							System.out.println("ADDED EMPTY 2: " + newP.print(newP.getEntities()));
						}
						//and create variation, where one nonterminal is replaced
						LinkedList<Entity> rhs = new LinkedList<Entity>();
						rhs.add(notEpsP.getEntities().get(0));
						//System.out.println("PRED ADD: " + addedProductions.size());
						addedProductions.add(new Production(notEpsP.getNonterminal(), rhs, false, new ProductionID() /*new ProductionID(notEpsP.getNonterminal())*/, notEpsP.getPriority()));
						//System.out.println("PO ADD: " + addedProductions.size());
						
					}
					//if is production "A -> B C ", where B xor C is epsilon production
					else if (notEpsP.getEntities().get(1).toString().equals(ep.getNonterminal())) {
						//notEpsP.getEntities().remove(1);
						ArrayList<Entity> rhs = new ArrayList<Entity>();
						rhs.add(notEpsP.getEntities().get(0));
						//System.out.println("PRED ADD: " + addedProductions.size());
						addedProductions.add(new Production(notEpsP.getNonterminal(), rhs, false, new ProductionID()/*new ProductionID(notEpsP.getNonterminal())*/, notEpsP.getPriority()));
						//System.out.println("PO ADD: " + addedProductions.size());
					}
					else if (notEpsP.getEntities().get(0).toString().equals(ep.getNonterminal())) {
						//notEpsP.getEntities().remove(0);
						ArrayList<Entity> rhs = new ArrayList<Entity>();
						rhs.add(notEpsP.getEntities().get(1));
						//System.out.println("PRED ADD: " + addedProductions.size());
						addedProductions.add(new Production(notEpsP.getNonterminal(), rhs, false, new ProductionID() /*new ProductionID(notEpsP.getNonterminal())*/, notEpsP.getPriority()));
						//System.out.println("PO ADD: " + addedProductions.size());
					}
					System.out.println("VELKOST 2 " + notEpsP.print(notEpsP.getEntities()));
				}
				else
					throw new IllegalArgumentException("grammar is not normalized");
			}
			
			//productions.addAll(addedProductions);
			for (Production p : addedProductions) {
				if (!productions.contains(p)) {
					System.out.println("ADDED " + p.print(p.getEntities()));
					productions.add(p);
				}
				else {
					//System.out.println("Don't added " + p.print(p.getEntities()));
				}
			}
			
//			//VYPIS
//			System.out.println("PRODUCTIONS:");
//			for (Production p : productions) {
//				System.out.println("P: " + p.print(p.getEntities()));
//			}
//			System.out.println("addedProductions");
//			for (Production p : addedProductions) {
//				System.out.println("ADDED PROD: " + p.print(p.getEntities()));
//			}
			
			addedProductions.clear();
			Production pp = epsilonProductions.pollFirst();
			usedEpsilonProduction.add(pp);
			//if we have epsilon porduction with start grammar symbol on left side, its OK, we add it to productions
			if (pp.getNonterminal().equals(ng.getStart()) && !productions.contains(pp)) {
				productions.add(pp);
			}
		}
		usedEpsilonProduction.clear();
		return new Grammar(ng.getStart(), productions);
	}

}
