package dk.brics.grammar.operations;

import java.util.ArrayList;
import java.util.List;

import dk.brics.grammar.EOFTerminalEntity;
import dk.brics.grammar.Entity;
import dk.brics.grammar.EntityVisitor;
import dk.brics.grammar.Grammar;
import dk.brics.grammar.NonterminalEntity;
import dk.brics.grammar.Production;
import dk.brics.grammar.RegexpTerminalEntity;
import dk.brics.grammar.StringTerminalEntity;

/**
 * Grammar tokenizer.
 * <p>
 * Replaces every regexp terminal by a string terminal.
 * Note that this is sound (for ambiguity analysis) only if the terminals represent tokens
 * such that every string can be split in at most one way into tokens.
 */
public class GrammarTokenizer {
	
	/**
	 * Constructs a new grammar tokenizer.
	 */
	public GrammarTokenizer() {}
	
	/**
	 * Tokenizes the given grammar.
	 * @param g grammar
	 */
	public void tokenize(Grammar g) {
		for (Production p : g.getProductions()) {
			List<Entity> newentities = new ArrayList<Entity>();
			for (Entity e : p.getEntities())
				newentities.add(e.visitBy(new EntityVisitor<Entity>() {
					@Override
					public Entity visitNonterminalEntity(NonterminalEntity e) {
						return e;
					}
					@Override
					public Entity visitRegexpTerminalEntity(RegexpTerminalEntity e) {
						return new StringTerminalEntity("$" + e.getAutomatonName());
					}
					@Override
					public Entity visitStringTerminalEntity(StringTerminalEntity e) {
						if (e.getString().startsWith("$"))
							return new StringTerminalEntity("@" + e.getString());
						else
							return e;
					}
					@Override
					public Entity visitEOFTerminalEntity(EOFTerminalEntity e) {
						return e;
					}
				}));
			p.setEntities(newentities);
		}
	}
}
