package dk.brics.grammar.operations;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import dk.brics.grammar.Entity;
import dk.brics.grammar.Grammar;
import dk.brics.grammar.NonterminalEntity;
import dk.brics.grammar.Production;
import dk.brics.grammar.ProductionID;

/**
 * Normalizes grammars to have at most two entities in each production.
 */
public class ProductionLengthNormalizer {
	
	/** 
	 * Print stream for messages. 
	 */
	private PrintWriter out;
	
	/** 
	 * Constructs a new normalizer.
	 * @param out print writer for messages (if a grammar contains unordered productions)
	 */
	public ProductionLengthNormalizer(PrintWriter out) {
		this.out = out;
	}
	
	/**
	 * Normalizes the given grammar.
	 * The input grammar is not modified, but the output grammar reuses the <code>Entity</code> objects from the input grammar.
	 * @param g input grammar
	 * @return output grammar
	 */
	public Grammar normalize(Grammar g) {
		int nextfresh = 1;
		Collection<Production> productions = new ArrayList<Production>();
		boolean contains_unordered = false;
		for (Production p : g.getProductions()) {
			if (p.isUnordered()) // FIXME: handle unordered productions? (e.g. A&B&C => (A|B|C)+ ??)
				contains_unordered = true;
			String nt = p.getNonterminal();
			String label = p.getID().getLabel();
			int priority = p.getPriority();
			List<Entity> oldent = p.getEntities();
			int i = 0;
			while (i+2 < oldent.size()) {
				String next = p.getNonterminal() + "'" + nextfresh++;
				NonterminalEntity n = new NonterminalEntity(next, "$", null);
				List<Entity> newent = new ArrayList<Entity>();
				newent.add(oldent.get(i));
				newent.add(n);
				productions.add(new Production(nt, newent, false, new ProductionID(label), priority));
				label = null;
				priority = 0;
				nt = next;
				i++;
			}
			List<Entity> newent = new ArrayList<Entity>();
			while (i < oldent.size()) {
				newent.add(oldent.get(i));
				i++;
			}
			productions.add(new Production(nt, newent, false, new ProductionID(label), priority));
		}
		if (contains_unordered)
			out.println("*** warning: grammar contains unordered productions (treated as ordered)");
		return new Grammar(g.getStart(), productions);
	}
}
