package dk.brics.grammar.operations;

import java.util.ArrayList;
import java.util.IdentityHashMap;
import java.util.List;
import java.util.Map;

import dk.brics.automaton.Automaton;
import dk.brics.grammar.Entity;
import dk.brics.grammar.Grammar;
import dk.brics.grammar.NonterminalEntity;
import dk.brics.grammar.Production;
import dk.brics.grammar.RegexpTerminalEntity;

/**
 * Normalizes grammars to have only one-transition-interval regexp terminal entities.
 * @see Automaton2Grammar
 */
public class RegexpTerminalNormalizer {

	/** 
	 * Constructs a new normalizer.
	 */
	public RegexpTerminalNormalizer() {}
	
	/**
	 * Normalizes the given grammar.
	 * @param g input grammar, modified
	 */
	public void normalize(Grammar g) {
		Automaton2Grammar a2g = new Automaton2Grammar();
		Map<Automaton,String> aut2nt = new IdentityHashMap<Automaton,String>();
		for (Production p : new ArrayList<Production>(g.getProductions()))  {
			List<Entity> newentities = new ArrayList<Entity>();
			for (Entity e : p.getEntities()) {
				if (e instanceof RegexpTerminalEntity) {
					RegexpTerminalEntity rt = (RegexpTerminalEntity)e;
					Automaton a = rt.getAutomaton();
					String nt = aut2nt.get(a);
					if (nt == null) {
						nt = a2g.extend(g, a);
						aut2nt.put(a, nt);
					}
					e = new NonterminalEntity(nt, e.getLabel(), e.getExample());
				}
				newentities.add(e);
			}
			p.setEntities(newentities);
		}
	}
}
