package dk.brics.grammar.operations;

import java.io.CharArrayWriter;
import java.io.PrintWriter;

import dk.brics.grammar.Entity;
import dk.brics.grammar.Grammar;
import dk.brics.grammar.ast.AST;
import dk.brics.grammar.ast.Node;
import dk.brics.grammar.ast.NodeVisitor;
import dk.brics.grammar.ast.BranchNode;
import dk.brics.grammar.ast.LeafNode;

/**
 * AST unparser.
 */
public class Unparser {

	private Grammar g;
	
	private ExampleStringDeriver deriver;
	
	/**
	 * Constructs a new AST unparser.
	 * @param g grammar
	 */
	public Unparser(Grammar g) {
		this.g = g;
		deriver = new ExampleStringDeriver(g);
	}
	
	/**
	 * Unparses the given AST.
	 * @param a AST
	 * @return unparsed AST
	 */
	public String unparse(AST a) {
		return unparse(a.getRoot(), a.getOriginalString());
	}
	
	/**
	 * Unparses the given AST.
	 * @param a AST
	 * @param p print writer for output
	 */
	public void unparse(AST a, PrintWriter p) {
		unparse(a.getRoot(), p, a.getOriginalString());
	}
	
	/**
	 * Unparses the given node representing a sub-AST.
	 * @param n node
	 * @param x original string
	 * @return unparsed sub-AST
	 */
	public String unparse(Node n, String x) {
		final CharArrayWriter b = new CharArrayWriter();
		unparse(n, new PrintWriter(b, true), x);
		return b.toString();
	}
	
	/**
	 * Unparses the given node representing a sub-AST.
	 * @param n node
	 * @param p print writer for output
	 * @param x original string
	 */
	public void unparse(Node n, final PrintWriter p, final String x) {
		n.visitBy(new NodeVisitor() {
			public void visitBranchNode(BranchNode n) {
				for (Entity e : g.getProduction(n.getProductionID()).getEntities())
					if (e.isExplicitlyLabeled())
						unparse(n.getChild(e.getLabel()), p, x);
					else if (e.getExample() != null)
						p.append(e.getExample());
					else 
						p.append(deriver.getExample(e));
			}
			public void visitLeafNode(LeafNode s) {
				p.append(s.getString(x));
			}
		});
	}
}
