package dk.brics.grammar.parser;

import java.io.Serializable;

/**
 * Representation of a location in a text.
 */
public class Location implements Serializable {

	private static final long serialVersionUID = 1L;
	
	/** Path or URL of file. */
	private String file;

	/** Character index. */
	private int index;
	
	/** Line number. */
	private int line;
	
	/** Column number. */
	private int col;
	
	/**
	 * Constructs a new location.
	 * @param file path or URL of text (null if unknown)
	 * @param contents contents of the text
	 * @param index char index into the text
	 */
	public Location(String file, String contents, int index) {
		this.file = file;
		this.index = index;
		setLineCol(contents);
	}
	
	/**
	 * Constructs a new location.
	 * @param file path or URL of text (null if unknown)
	 * @param index char index
	 * @param line line number
	 * @param col column number
	 */
	public Location(String file, int index, int line, int col) {
		this.file = file;
		this.index = index;
		this.line = line;
		this.col = col;
	}
	
	private void setLineCol(String contents) {
		line = col = 1;
		for (int n = 0; n < index; n++) {
			char c = contents.charAt(n);
			if (c == '\n' || c == '\r') {
				line++;
				col = 1;
				if (n + 1 < index && c == '\r' && contents.charAt(n + 1) == '\n')
					n++;
			} else if (c == '\t') 
				do 
					col++;
				while (col % 8 != 1); // assuming tab size 8
			else
				col++;
		}
	}

	/**
	 * Returns the character index.
	 * @return character index
	 */
	public int getIndex() {
		return index;
	}
	
	/**
	 * Returns the file name.
	 * @return path or URL
	 */
	public String getFile() {
		return file;
	}

	/**
	 * Returns the line number.
	 * @return char line number (counting from 1)
	 */
	public int getLine() {
		return line;
	}

	/**
	 * Returns the column number.
	 * @return column number (counting from 1, tab size is assumed to be 8)
	 */
	public int getColumn() {
		return col;
	}
}
