package dk.brics.grammar.parser;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import dk.brics.automaton.Automaton;
import dk.brics.automaton.RegExp;
import dk.brics.grammar.EOFTerminalEntity;
import dk.brics.grammar.Entity;
import dk.brics.grammar.Grammar;
import dk.brics.grammar.NonterminalEntity;
import dk.brics.grammar.Production;
import dk.brics.grammar.ProductionID;
import dk.brics.grammar.RegexpTerminalEntity;
import dk.brics.grammar.StringTerminalEntity;

/**
 * Grammar for grammars.
 */
public class MetaGrammar {

	/** Cached meta-grammar. */
	static Grammar meta;
	
	private MetaGrammar() {}

	/**
	 * Returns meta grammar. 
	 * (Corresponds to <a target="_top" href="../../../../../meta.cfg">meta.cfg</a>.)
	 * @return meta grammar
	 */
	static public Grammar getMetaGrammar() {
		if (meta == null) {
			
			Automaton a_id = new RegExp("[a-zA-Z0-9_\\.]+").toAutomaton();
			String a_id_name = "ID";
			Automaton a_whitespace = new RegExp("[\\ \t\n\r\f]+").toAutomaton();
			String a_whitespace_name = "WHITESPACE";
			Automaton a_comment = new RegExp("\"/*\"~(.*\"*/\".*)\"*/\"|\"//\"[^\n\r]*").toAutomaton();
			String a_comment_name = "COMMENT";
			Automaton a_number = new RegExp("[0-9]+").toAutomaton();
			String a_number_name = "NUMBER";
			Automaton a_nonreservedchar = new RegExp("[^\\ \\.\\\"\\#\\@\\|\\&\\~\\?\\*\\+\\{\\}\\(\\)\\<\\>\\[\\]\\-\\\\\b\t\n\r\f]").toAutomaton();
			String a_nonreservedchar_name = "NONRESERVEDCHAR";
			Automaton a_escapedchar = new RegExp("\\\\(u[0-9a-fA-F]{4}|[^u\b\t\n\r\f]|[btnrf])").toAutomaton();
			String a_escapedchar_name = "ESCAPEDCHAR";
			Automaton a_charclasschar = new RegExp("[^\\^\\]\\\\\\-\b\t\n\r\f]").toAutomaton();
			String a_charclasschar_name = "CHARCLASSCHAR";
			Automaton a_string = new RegExp("([^\"\\\\\b\t\n\r\f]|\\\\(u[0-9a-fA-F]{4}|[^u\b\t\n\r\f]|[btnrf]))*").toAutomaton();
			String a_string_name = "STRING";
			
			Collection<Production> productions = new ArrayList<Production>();
			List<Entity> e;
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("_", null, null));
			e.add(new NonterminalEntity("RegexpDefs", "regexps", null));
			e.add(new NonterminalEntity("Productions", "productions", null));
			productions.add(new Production("Grammar", e, false, new ProductionID(), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("RegexpDef", "regexp", null));
			e.add(new NonterminalEntity("__", null, null));
			e.add(new NonterminalEntity("RegexpDefs", "more", null));
			productions.add(new Production("RegexpDefs", e, false, new ProductionID("nonempty"), 0));
			
			e = new ArrayList<Entity>();
			productions.add(new Production("RegexpDefs", e, false, new ProductionID("empty"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new RegexpTerminalEntity(a_id, true, a_id_name, "name", null));
			e.add(new NonterminalEntity("_", null, null));
			e.add(new StringTerminalEntity("="));
			e.add(new NonterminalEntity("_", null, null));
			e.add(new NonterminalEntity("Regexp", "exp", null));
			e.add(new NonterminalEntity("OptMax", "max", null));
			productions.add(new Production("RegexpDef", e, false, new ProductionID(), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("ProductionGroup", "production", null));
			e.add(new NonterminalEntity("Productions", "more", null));
			productions.add(new Production("Productions", e, false, new ProductionID("nonempty"), 0));
			
			e = new ArrayList<Entity>();
			productions.add(new Production("Productions", e, false, new ProductionID("empty"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new RegexpTerminalEntity(a_id, true, a_id_name, "nonterminal", null));
			e.add(new NonterminalEntity("_", null, null));
			e.add(new NonterminalEntity("OptLabel", "label", null));
			e.add(new NonterminalEntity("OptPriority", "priority", null));
			e.add(new StringTerminalEntity(":"));
			e.add(new NonterminalEntity("OptUnordered", "unordered", null));
			e.add(new NonterminalEntity("Entities", "entities", null));
			e.add(new NonterminalEntity("__", null, null));
			e.add(new NonterminalEntity("MoreProductions", "more", null));
			productions.add(new Production("ProductionGroup", e, false, new ProductionID(), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("OptLabel", "label", null));
			e.add(new NonterminalEntity("OptPriority", "priority", null));
			e.add(new StringTerminalEntity("|"));
			e.add(new NonterminalEntity("OptUnordered", "unordered", null));
			e.add(new NonterminalEntity("Entities", "entities", null));
			e.add(new NonterminalEntity("__", null, null));
			e.add(new NonterminalEntity("MoreProductions", "more", null));
			productions.add(new Production("MoreProductions", e, false, new ProductionID("nonempty"), 0));
			
			e = new ArrayList<Entity>();
			productions.add(new Production("MoreProductions", e, false, new ProductionID("empty"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new StringTerminalEntity("["));
			e.add(new NonterminalEntity("_", null, null));
			e.add(new RegexpTerminalEntity(a_id, true, a_id_name, "label", null));
			e.add(new NonterminalEntity("_", null, null));
			e.add(new StringTerminalEntity("]"));
			e.add(new NonterminalEntity("_", null, null));
			productions.add(new Production("OptLabel", e, false, new ProductionID("present"), 0));
			
			e = new ArrayList<Entity>();
			productions.add(new Production("OptLabel", e, false, new ProductionID("absent"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new StringTerminalEntity(">"));
			e.add(new NonterminalEntity("_", null, null));
			productions.add(new Production("OptPriority", e, false, new ProductionID("higher"), 0));
			
			e = new ArrayList<Entity>();
			productions.add(new Production("OptPriority", e, false, new ProductionID("same"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("_", null, null));
			e.add(new StringTerminalEntity("&"));
			productions.add(new Production("OptUnordered", e, false, new ProductionID("present"), 0));
			
			e = new ArrayList<Entity>();
			productions.add(new Production("OptUnordered", e, false, new ProductionID("absent"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("_", null, null));
			e.add(new NonterminalEntity("Entity", "entity", null));
			e.add(new NonterminalEntity("Entities", "more", null));
			productions.add(new Production("Entities", e, false, new ProductionID("nonempty"), 0));
			
			e = new ArrayList<Entity>();
			productions.add(new Production("Entities", e, false, new ProductionID("empty"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new RegexpTerminalEntity(a_id, true, a_id_name, "nonterminal", null));
			e.add(new NonterminalEntity("OptLabelOrExample", "labelexample", null));
			productions.add(new Production("Entity", e, false, new ProductionID("nonterminal"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new StringTerminalEntity("<"));
			e.add(new RegexpTerminalEntity(a_id, true, a_id_name, "regexp", null));
			e.add(new StringTerminalEntity(">"));
			e.add(new NonterminalEntity("OptLabelOrExample", "labelexample", null));
			productions.add(new Production("Entity", e, false, new ProductionID("regexp_terminal"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new StringTerminalEntity("\""));
			e.add(new RegexpTerminalEntity(a_string, true, a_string_name, "string", null));
			e.add(new StringTerminalEntity("\""));
			productions.add(new Production("Entity", e, false, new ProductionID("string_terminal"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new StringTerminalEntity("["));
			e.add(new NonterminalEntity("_", null, null));
			e.add(new RegexpTerminalEntity(a_id, true, a_id_name, "label", null));
			e.add(new NonterminalEntity("_", null, null));
			e.add(new StringTerminalEntity("]"));
			productions.add(new Production("OptLabelOrExample", e, false, new ProductionID("label"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new StringTerminalEntity("["));
			e.add(new NonterminalEntity("_", null, null));
			e.add(new StringTerminalEntity("\""));
			e.add(new RegexpTerminalEntity(a_string, true, a_string_name, "example", null));
			e.add(new StringTerminalEntity("\""));
			e.add(new NonterminalEntity("_", null, null));
			e.add(new StringTerminalEntity("]"));
			productions.add(new Production("OptLabelOrExample", e, false, new ProductionID("example"), 0));
			
			e = new ArrayList<Entity>();
			productions.add(new Production("OptLabelOrExample", e, false, new ProductionID("absent"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("UnionExp", "e", null));
			productions.add(new Production("Regexp", e, false, new ProductionID(), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("InterExp", "e", null));
			e.add(new StringTerminalEntity("|"));
			e.add(new NonterminalEntity("UnionExp", "more", null));
			productions.add(new Production("UnionExp", e, false, new ProductionID("union"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("InterExp", "e", null));
			productions.add(new Production("UnionExp", e, false, new ProductionID("other"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("ConcatExp", "e", null));
			e.add(new StringTerminalEntity("&"));
			e.add(new NonterminalEntity("InterExp", "more", null));
			productions.add(new Production("InterExp", e, false, new ProductionID("inter"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("ConcatExp", "e", null));
			productions.add(new Production("InterExp", e, false, new ProductionID("other"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("RepeatExp", "e", null));
			e.add(new NonterminalEntity("ConcatExp", "more", null));
			productions.add(new Production("ConcatExp", e, false, new ProductionID("concat"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("RepeatExp", "e", null));
			productions.add(new Production("ConcatExp", e, false, new ProductionID("other"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("RepeatExp", "e", null));
			e.add(new StringTerminalEntity("?"));
			productions.add(new Production("RepeatExp", e, false, new ProductionID("optional"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("RepeatExp", "e", null));
			e.add(new StringTerminalEntity("*"));
			productions.add(new Production("RepeatExp", e, false, new ProductionID("star"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("RepeatExp", "e", null));
			e.add(new StringTerminalEntity("+"));
			productions.add(new Production("RepeatExp", e, false, new ProductionID("plus"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("RepeatExp", "e", null));
			e.add(new StringTerminalEntity("{"));
			e.add(new RegexpTerminalEntity(a_number, true, a_number_name, "n", null));
			e.add(new StringTerminalEntity("}"));
			productions.add(new Production("RepeatExp", e, false, new ProductionID("number"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("RepeatExp", "e", null));
			e.add(new StringTerminalEntity("{"));
			e.add(new RegexpTerminalEntity(a_number, true, a_number_name, "n", null));
			e.add(new StringTerminalEntity(",}"));
			productions.add(new Production("RepeatExp", e, false, new ProductionID("min"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("RepeatExp", "e", null));
			e.add(new StringTerminalEntity("{"));
			e.add(new RegexpTerminalEntity(a_number, true, a_number_name, "n", null));
			e.add(new StringTerminalEntity(","));
			e.add(new RegexpTerminalEntity(a_number, true, a_number_name, "m", null));
			e.add(new StringTerminalEntity("}"));
			productions.add(new Production("RepeatExp", e, false, new ProductionID("interval"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("ComplExp", "e", null));
			productions.add(new Production("RepeatExp", e, false, new ProductionID("other"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new StringTerminalEntity("~"));
			e.add(new NonterminalEntity("ComplExp", "e", null));
			productions.add(new Production("ComplExp", e, false, new ProductionID("complement"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("CharclassExp", "e", null));
			productions.add(new Production("ComplExp", e, false, new ProductionID("other"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new StringTerminalEntity("["));
			e.add(new NonterminalEntity("Charclasses", "c", null));
			e.add(new StringTerminalEntity("]"));
			productions.add(new Production("CharclassExp", e, false, new ProductionID("charclass"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new StringTerminalEntity("[^"));
			e.add(new NonterminalEntity("Charclasses", "c", null));
			e.add(new StringTerminalEntity("]"));
			productions.add(new Production("CharclassExp", e, false, new ProductionID("negativeclass"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("SimpleExp", "e", null));
			productions.add(new Production("CharclassExp", e, false, new ProductionID("other"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("Charclass", "c", null));
			e.add(new NonterminalEntity("Charclasses", "more", null));
			productions.add(new Production("Charclasses", e, false, new ProductionID("first"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("Charclass", "c", null));
			productions.add(new Production("Charclasses", e, false, new ProductionID("last"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("Charclasschar", "c1", null));
			e.add(new StringTerminalEntity("-"));
			e.add(new NonterminalEntity("Charclasschar", "c2", null));
			productions.add(new Production("Charclass", e, false, new ProductionID("interval"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("Charclasschar", "c", null));
			productions.add(new Production("Charclass", e, false, new ProductionID("single"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new RegexpTerminalEntity(a_charclasschar, false, a_charclasschar_name, "c", null));
			productions.add(new Production("Charclasschar", e, false, new ProductionID("char"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new RegexpTerminalEntity(a_escapedchar, false, a_escapedchar_name, "c", null));
			productions.add(new Production("Charclasschar", e, false, new ProductionID("escape"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new RegexpTerminalEntity(a_nonreservedchar, false, a_nonreservedchar_name, "c", null));
			productions.add(new Production("SimpleExp", e, false, new ProductionID("char"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new RegexpTerminalEntity(a_escapedchar, false, a_escapedchar_name, "c", null));
			productions.add(new Production("SimpleExp", e, false, new ProductionID("escape"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new StringTerminalEntity("."));
			productions.add(new Production("SimpleExp", e, false, new ProductionID("dot"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new StringTerminalEntity("#"));
			productions.add(new Production("SimpleExp", e, false, new ProductionID("empty"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new StringTerminalEntity("@"));
			productions.add(new Production("SimpleExp", e, false, new ProductionID("all"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new StringTerminalEntity("\""));
			e.add(new RegexpTerminalEntity(a_string, true, a_string_name, "string", null));
			e.add(new StringTerminalEntity("\""));
			productions.add(new Production("SimpleExp", e, false, new ProductionID("string"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new StringTerminalEntity("()"));
			productions.add(new Production("SimpleExp", e, false, new ProductionID("epsilon"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new StringTerminalEntity("("));
			e.add(new NonterminalEntity("UnionExp", "e", null));
			e.add(new StringTerminalEntity(")"));
			productions.add(new Production("SimpleExp", e, false, new ProductionID("exp"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new StringTerminalEntity("<"));
			e.add(new RegexpTerminalEntity(a_id, true, a_id_name, "id", null));
			e.add(new StringTerminalEntity(">"));
			productions.add(new Production("SimpleExp", e, false, new ProductionID("named"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new StringTerminalEntity("<"));
			e.add(new RegexpTerminalEntity(a_number, true, a_number_name, "n", null));
			e.add(new StringTerminalEntity("-"));
			e.add(new RegexpTerminalEntity(a_number, true, a_number_name, "m", null));
			e.add(new StringTerminalEntity(">"));
			productions.add(new Production("SimpleExp", e, false, new ProductionID("numeric"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("__", null, null));
			e.add(new StringTerminalEntity("("));
			e.add(new StringTerminalEntity("MAX"));
			e.add(new StringTerminalEntity(")"));
			productions.add(new Production("OptMax", e, false, new ProductionID("present"), 0));
			
			e = new ArrayList<Entity>();
			productions.add(new Production("OptMax", e, false, new ProductionID("absent"), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("Ignorable", null, null));
			e.add(new NonterminalEntity("_", null, null));
			productions.add(new Production("_", e, false, new ProductionID(), 0));
			
			e = new ArrayList<Entity>();
			productions.add(new Production("_", e, false, new ProductionID(), 0));
			
			e = new ArrayList<Entity>();
			e.add(new NonterminalEntity("Ignorable", null, null));
			e.add(new NonterminalEntity("_", null, null));
			productions.add(new Production("__", e, false, new ProductionID(), 0));
			
			e = new ArrayList<Entity>();
			e.add(new RegexpTerminalEntity(a_whitespace, true, a_whitespace_name, null, null));
			productions.add(new Production("Ignorable", e, false, new ProductionID(), 0));
			
			e = new ArrayList<Entity>();
			e.add(new RegexpTerminalEntity(a_comment, true, a_comment_name, null, null));
			productions.add(new Production("Ignorable", e, false, new ProductionID(), 0));
			
			e = new ArrayList<Entity>();
			e.add(new EOFTerminalEntity());
			productions.add(new Production("Ignorable", e, false, new ProductionID(), 0));
			
			meta = new Grammar("Grammar", productions);
		}
		return meta;
	}
}
