package dk.brics.grammar.parser;

/**
 * Parse error exception.
 */
public class ParseException extends Exception {

	private static final long serialVersionUID = 1L;
	
	Location loc;
	
	ParseException(String file, String x, int i) {
		super("parse error at character " + i);
		loc = new Location(file, x, i);
	}
	
	/**
	 * Returns the detail message string
	 * @return exception message
	 */
	@Override
	public String getMessage() {
		String msg;
		if (loc.getIndex() >= 0)
			msg =  "parse error at character " + loc.getIndex() +  " (line " + loc.getLine() + ", column " + loc.getColumn() + ")";
		else
			msg = "parse error (line " + loc.getLine() + ", column " + loc.getColumn() + ")";
		if (loc.getFile() != null)
			msg += " in " + loc.getFile();
		return msg;
	}
	
	/**
	 * Sets the location.
	 * @param loc location
	 */
	public void setLocation(Location loc) {
		this.loc = loc;
	}
	
	/**
	 * Returns the location where the parser got stuck.
	 * @return location
	 */
	public Location getLocation() {
		return loc;
	}
}
