package dk.brics.misc;

/**
 * Character escaping.
 */
public class Chars {
	
	private Chars() {}
	
	/**
	 * Escapes special characters in the given string.
	 * Same as {@link #escape(String, boolean) escape(s, true)}.
	 */
	public static String escape(String s) {
		return escape(s, false);
	}

	/**
	 * Escapes special characters in the given string.
	 * @param dot if true, use Graphviz dot mode
	 */
	public static String escape(String s, boolean dot) {
		StringBuilder b = new StringBuilder();
		for (int i = 0; i < s.length(); i++) {
			char c = s.charAt(i);
			switch (c) {
			case '"':
				b.append("\\\"");
				break;
			case '\\':
				b.append("\\\\");
				break;
			case '\b':
				b.append("\\b");
				break;
			case '\t':
				b.append("\\t");
				break;
			case '\n':
				b.append("\\n");
				break;
			case '\r':
				b.append("\\r");
				break;
			case '\f':
				b.append("\\f");
				break;
			default:
				if (c >= 0x20 && c <= 0x7e && c != '\\' && c != '"')
					b.append(c);
				else {
					b.append("\\u");
					String t = Integer.toHexString(c & 0xffff);
					for (int j = 0; j + t.length() < 4; j++)
						b.append('0');
					b.append(t);
				}
			}
		}
		return b.toString();
	}
}
