package dk.brics.misc;

/**
 * System properties.
 */
public class Properties {

	private Properties() {}
	
	/**
	 * Checks whether the given system property is set.
	 * The values <code>false</code> and <code>0</code> are here equivalent to "unset".
	 * @param p property name
	 * @return property value
	 */
	public static boolean get(String p) {
		String s = System.getProperty(p);
		if (s != null) {
			if (s.equals("false") || s.equals("0"))
				return false;
			else
				return true;
		}
		return false;
	}
	
	/**
	 * Returns list of strings from the given property using comma as delimiter.
	 * @param p property name
	 * @return list of string values, null if none
	 */
	public static String[] getStrings(String p) {
		String s = System.getProperty(p);
		if (s == null)
			return null;
		else
			return s.split(",");
	}
}
